@@ -516,6 +516,8 @@ class Value:
 
     dtype: str  # must be initialized when created
     description: Optional[str] = None
+    max_value: Optional[float | int] = None
+    min_value: Optional[float | int] = None
     is_bucket: bool = False  # don't need to be initialized
     require_training_set: bool = False
     is_pre_computed: bool = False
