@@ -5,6 +5,7 @@ import dataclasses
 from dataclasses import asdict, dataclass, field
 import json
 import os
+import sys
 from typing import Any, Optional
 
 from explainaboard import config
@@ -89,13 +90,13 @@ class SysOutputInfo:
     model_name: Optional[str] = None
     dataset_name: Optional[str] = None
     sub_dataset_name: Optional[str] = None
-    metric_names: Optional[list[str]] = None
+    dataset_split: Optional[str] = None
     language: str = "en"
     reload_stat: bool = True
     is_print_case: bool = True
     conf_value: float = 0.05
     system_details: Optional[dict] = None
-    metric_configs: Optional[dict[str, MetricConfig]] = None
+    metric_configs: Optional[list[MetricConfig]] = None
     tokenizer: Tokenizer = SingleSpaceTokenizer()
 
     # set later
@@ -137,11 +138,13 @@ class SysOutputInfo:
         with open(file_path, "wb") as f:
             self._dump_info(f)
 
-    def print_as_json(self):
+    def print_as_json(self, file=None):
+        if file is None:
+            file = sys.stdout
         data_dict = self.to_dict()
         self.replace_nonstring_keys(data_dict)
         try:
-            print(json.dumps(data_dict, indent=2, default=lambda x: x.json_repr()))
+            json.dump(data_dict, fp=file, indent=2, default=lambda x: x.json_repr())
         except TypeError as e:
             raise e
 
