@@ -4,14 +4,16 @@ import abc
 from collections import Counter
 from dataclasses import dataclass
 import itertools
+import sys
 from typing import Any, cast, Optional, TypeVar, Union
 
 from eaas.async_client import AsyncRequest
+from eaas.endpoint import EndpointConfig
 import numpy as np
 import sacrebleu
 
 from explainaboard.utils.preprocessor import Preprocessor, QAPreprocessor
-from explainaboard.utils.span_utils import get_spans_from_bio
+from explainaboard.utils.span_utils import BIOSpanOps, BMESSpanOps, Span
 from explainaboard.utils.typing_utils import unwrap
 
 T = TypeVar('T')
@@ -23,8 +25,6 @@ class MetricResult:
     A result of computing a metric over some data
     """
 
-    # Name of the metric that was computed
-    name: str
     # Configuration with which it was calculated
     config: MetricConfig
     # Metric value
@@ -36,7 +36,6 @@ class MetricResult:
 
     def to_dict(self):
         ret = {
-            'name': self.name,
             'config': self.config.__dict__,
             'value': self.value,
         }
@@ -55,8 +54,12 @@ class MetricConfig(dict):
     or when performing individual metric computation.
     """
 
+    name: str
     language: str = "en"
 
+    def to_metric(self):
+        raise NotImplementedError
+
 
 class MetricStats:
     """
@@ -100,25 +103,15 @@ class Metric:
     A class representing an evaluation metric
     """
 
-    @classmethod
-    @abc.abstractmethod
-    def default_name(cls) -> str:
-        """Returns the default name of the metric."""
-        ...
-
     def __init__(
         self,
-        name: Optional[str] = None,
-        config: Optional[MetricConfig] = None,
+        config: MetricConfig,
     ):
         """
         Initialize the metric
-        :param name: the name of the metric for reference later
+        :param config: The configuration for the metric
         """
-        self.name: str = unwrap(name) if name is not None else self.default_name()
-        self.config: MetricConfig = (
-            unwrap(config) if config is not None else MetricConfig()
-        )
+        self.config: MetricConfig = config
 
     def _get_config(self, config: Optional[MetricConfig] = None) -> MetricConfig:
         """
@@ -151,7 +144,12 @@ class Metric:
         :param stats: stats for every example
         :return: aggregated stats
         """
-        return np.mean(stats.get_data(), axis=0)
+
+        data = stats.get_data()
+        if data.size == 0:
+            return np.array(0.0)
+        else:
+            return np.mean(data, axis=0)
 
     def calc_metric_from_aggregate(
         self, agg_stats: np.ndarray, config: Optional[MetricConfig] = None
@@ -170,7 +168,7 @@ class Metric:
         self,
         stats: MetricStats,
         conf_value: float,
-        n_samples: int = 2000,
+        n_samples: int = 1000,
         prop_samples: float = 0.5,
         config: Optional[MetricConfig] = None,
     ) -> tuple[float, float]:
@@ -186,8 +184,10 @@ class Metric:
         n_elems = int(prop_samples * len(stats))
         samp_results = np.zeros(shape=(n_samples,))
         all_indices = np.array(range(len(stats)))
+        rng = np.random.default_rng()
+        all_indices = rng.choice(all_indices, size=(n_samples, n_elems), replace=True)
         for i in range(n_samples):
-            indices = np.random.choice(all_indices, size=n_elems, replace=True)
+            indices = all_indices[i]
             agg_stats = self.aggregate_stats(stats.filter(indices))
             samp_results[i] = self.calc_metric_from_aggregate(agg_stats, config)
         samp_results = np.sort(samp_results)
@@ -214,7 +214,7 @@ class Metric:
         conf_interval = (
             self.bootstrap_interval(stats, conf_value) if conf_value else None
         )
-        return MetricResult(self.name, config, value, conf_interval, conf_value)
+        return MetricResult(config, value, conf_interval, conf_value)
 
     def evaluate(
         self,
@@ -235,16 +235,18 @@ class Metric:
         return self.evaluate_from_stats(stats, conf_value, config)
 
 
+@dataclass
+class AccuracyConfig(MetricConfig):
+    def to_metric(self):
+        return Accuracy(self)
+
+
 class Accuracy(Metric):
     """
     Calculate zero-one accuracy, where score is 1 iff the prediction equals the ground
     truth
     """
 
-    @classmethod
-    def default_name(cls) -> str:
-        return 'Accuracy'
-
     def calc_stats_from_data(
         self, true_data: list, pred_data: list, config: Optional[MetricConfig] = None
     ) -> MetricStats:
@@ -259,6 +261,9 @@ class F1ScoreConfig(MetricConfig):
     separate_match: bool = False
     ignore_classes: Optional[list] = None
 
+    def to_metric(self):
+        return F1Score(self)
+
 
 class F1Score(Metric):
     """
@@ -266,24 +271,6 @@ class F1Score(Metric):
     implementation.
     """
 
-    def __init__(
-        self,
-        name: Optional[str] = None,
-        config: Optional[MetricConfig] = None,
-    ):
-        """
-        Initialize the metric
-        :param name: the name of the metric for reference later
-        """
-        self.config: MetricConfig = (
-            unwrap(config) if config is not None else F1ScoreConfig()
-        )
-        super().__init__(name=name, config=self.config)
-
-    @classmethod
-    def default_name(cls) -> str:
-        return 'F1'
-
     def calc_stats_from_data(
         self, true_data: list, pred_data: list, config: Optional[MetricConfig] = None
     ) -> MetricStats:
@@ -307,6 +294,7 @@ class F1Score(Metric):
         if config.ignore_classes is not None:
             for ignore_class in config.ignore_classes:
                 id_map[ignore_class] = -1
+
         for word in itertools.chain(true_data, pred_data):
             if word not in id_map:
                 id_map[word] = len(id_map)
@@ -329,12 +317,18 @@ class F1Score(Metric):
     def calc_metric_from_aggregate(
         self, agg_stats: np.ndarray, config: Optional[MetricConfig] = None
     ) -> float:
+
+        if agg_stats.size == 1:
+            return float(agg_stats)
+
+        # print(agg_stats)
         config = cast(F1ScoreConfig, self._get_config(config))
         supported_averages = {'micro', 'macro'}
         stat_mult: int = 4 if config.separate_match else 3
         if config.average not in supported_averages:
             raise ValueError(f'only {supported_averages} supported for now')
         n_classes = int(len(agg_stats) / stat_mult)
+
         if config.average == 'micro':
             true, pred, true_match, pred_match = 0.0, 0.0, 0.0, 0.0
             for i in range(n_classes):
@@ -345,6 +339,7 @@ class F1Score(Metric):
             p = pred_match / pred if pred else 0.0
             r = true_match / true if true else 0.0
             f1_total = 2 * p * r / (p + r) if p + r != 0.0 else 0.0
+
         elif config.average == 'macro':
             f1_total = 0.0
             for i in range(n_classes):
@@ -360,6 +355,12 @@ class F1Score(Metric):
         return f1_total
 
 
+@dataclass
+class BIOF1ScoreConfig(F1ScoreConfig):
+    def to_metric(self):
+        return BIOF1Score(self)
+
+
 class BIOF1Score(F1Score):
     """
     Calculate F1 score over BIO-tagged spans.
@@ -367,8 +368,8 @@ class BIOF1Score(F1Score):
 
     def calc_stats_from_data(
         self,
-        true_data: list[list[str]],
-        pred_data: list[list[str]],
+        true_data: list[list[str]] | list[list[Span]],
+        pred_data: list[list[str]] | list[list[Span]],
         config: Optional[MetricConfig] = None,
     ) -> MetricStats:
         """
@@ -382,32 +383,145 @@ class BIOF1Score(F1Score):
             * c*stat_mult + 1: occurrences in the predicted output
             * c*stat_mult + 2: number of matches with the true output
         """
-        stat_mult = 3
 
-        # Identify the tag types
-        true_chain, pred_chain = (
-            itertools.chain.from_iterable(x) for x in (true_data, pred_data)
+        # 1. Span extraction
+        bio_span_ops = BIOSpanOps()
+        true_spans_list: list[list[Span]] = (
+            [
+                bio_span_ops.get_spans(true_tags)
+                for true_tags in true_data  # type: ignore
+            ]
+            if len(true_data[0]) == 0 or isinstance(true_data[0][0], Span) is False
+            else true_data
         )
-        all_tags = set(itertools.chain(true_chain, pred_chain))
-        tag_ids = {
-            k: v for v, k in enumerate([x[2:] for x in all_tags if x.startswith('B-')])
-        }
+        pred_spans_list: list[list[Span]] = (
+            [
+                bio_span_ops.get_spans(pred_tags)  # type: ignore
+                for pred_tags in pred_data
+            ]
+            if len(pred_data[0]) == 0 or isinstance(pred_data[0][0], Span) is False
+            else pred_data
+        )
+
+        # 2. Get tag space
+        all_tags = set(
+            [
+                span.span_tag  # type: ignore
+                for span in list(itertools.chain.from_iterable(true_spans_list))
+                + list(itertools.chain.from_iterable(pred_spans_list))
+            ]
+        )
+        tag_ids = {k: v for v, k in enumerate([x for x in all_tags])}
 
         # Create the sufficient statistics
+        stat_mult = 3
         n_data, n_classes = len(true_data), len(tag_ids)
         # This is a bit memory inefficient if there's a large number of classes
         stats = np.zeros((n_data, n_classes * stat_mult))
 
-        for i, (true_sent, pred_sent) in enumerate(zip(true_data, pred_data)):
-            true_spans, pred_spans = (
-                get_spans_from_bio(x) for x in (true_sent, pred_sent)
-            )
-            match_spans = [x for x in true_spans if x in pred_spans]
-            for offset, spans in enumerate((true_spans, pred_spans, match_spans)):
-                for chunk in spans:
-                    c = tag_ids[chunk[0]]
-                    stats[i, c * 3 + offset] += 1
+        for i, (true_spans, pred_spans) in enumerate(
+            zip(true_spans_list, pred_spans_list)
+        ):
 
+            (
+                matched_a_index,
+                matched_b_index,
+                matched_spans,
+                _,
+            ) = bio_span_ops.get_matched_spans(
+                true_spans,
+                pred_spans,
+            )
+
+            for offset, spans in enumerate((true_spans, pred_spans, matched_spans)):
+                for span in spans:
+                    c = tag_ids[span.span_tag]
+                    stats[i, c * stat_mult + offset] += 1
+        return MetricStats(stats)
+
+
+@dataclass
+class BMESF1ScoreConfig(F1ScoreConfig):
+    def to_metric(self):
+        return BMESF1Score(self)
+
+
+class BMESF1Score(F1Score):
+    """
+    Calculate F1 score over BIO-tagged spans.
+    """
+
+    def calc_stats_from_data(
+        self,
+        true_data: list[list[str]] | list[list[Span]],
+        pred_data: list[list[str]] | list[list[Span]],
+        config: Optional[MetricConfig] = None,
+    ) -> MetricStats:
+        """
+        Return sufficient statistics necessary to compute f-score.
+        :param true_data: True outputs
+        :param pred_data: Predicted outputs
+        :param config: Configuration, if over-riding the default
+        :return: Returns stats for each class (integer id c) in the following columns of
+            MetricStats
+            * c*stat_mult + 0: occurrences in the true output
+            * c*stat_mult + 1: occurrences in the predicted output
+            * c*stat_mult + 2: number of matches with the true output
+        """
+
+        # 1. Span extraction
+        bmes_span_ops = BMESSpanOps()
+        true_spans_list: list[list[Span]] = (
+            [
+                bmes_span_ops.get_spans(true_tags)
+                for true_tags in true_data  # type: ignore
+            ]
+            if len(true_data[0]) == 0 or isinstance(true_data[0][0], Span) is False
+            else true_data
+        )
+        pred_spans_list: list[list[Span]] = (
+            [
+                bmes_span_ops.get_spans(pred_tags)  # type: ignore
+                for pred_tags in pred_data
+            ]
+            if len(pred_data[0]) == 0 or isinstance(pred_data[0][0], Span) is False
+            else pred_data
+        )
+
+        # 2. Get tag space
+        all_tags = set(
+            [
+                span.span_tag  # type: ignore
+                for span in list(itertools.chain.from_iterable(true_spans_list))
+                + list(itertools.chain.from_iterable(pred_spans_list))
+            ]
+        )
+        tag_ids = {k: v for v, k in enumerate([x for x in all_tags])}
+
+        # Create the sufficient statistics
+        stat_mult = 3
+        n_data, n_classes = len(true_data), len(tag_ids)
+        # This is a bit memory inefficient if there's a large number of classes
+        stats = np.zeros((n_data, n_classes * stat_mult))
+
+        for i, (true_spans, pred_spans) in enumerate(
+            zip(true_spans_list, pred_spans_list)
+        ):
+
+            (
+                matched_a_index,
+                matched_b_index,
+                matched_spans,
+                _,
+            ) = bmes_span_ops.get_matched_spans(
+                true_spans,
+                pred_spans,
+            )
+
+            for offset, spans in enumerate((true_spans, pred_spans, matched_spans)):
+                for span in spans:
+                    c = tag_ids[span.span_tag]
+                    stats[i, c * stat_mult + offset] += 1
         return MetricStats(stats)
 
 
@@ -415,6 +529,9 @@ class BIOF1Score(F1Score):
 class HitsConfig(MetricConfig):
     hits_k: int = 5
 
+    def to_metric(self):
+        return Hits(self)
+
 
 class Hits(Metric):
     """
@@ -422,24 +539,6 @@ class Hits(Metric):
     outputs.
     """
 
-    def __init__(
-        self,
-        name: Optional[str] = None,
-        config: Optional[MetricConfig] = None,
-    ):
-        """
-        Initialize the metric
-        :param name: the name of the metric for reference later
-        """
-        self.config: MetricConfig = (
-            unwrap(config) if config is not None else HitsConfig()
-        )
-        super().__init__(name=name, config=self.config)
-
-    @classmethod
-    def default_name(cls) -> str:
-        return 'Hits'
-
     def calc_stats_from_data(
         self, true_data: list, pred_data: list, config: Optional[MetricConfig] = None
     ) -> MetricStats:  # TODO(Pengfei): why do we need the 3rd argument?
@@ -454,6 +553,12 @@ class Hits(Metric):
         )
 
 
+@dataclass
+class MeanReciprocalRankConfig(MetricConfig):
+    def to_metric(self):
+        return MeanReciprocalRank(self)
+
+
 class MeanReciprocalRank(Metric):
     """
     Calculates the mean reciprocal rank, 1/rank(true_output) where rank(true_output) is
@@ -479,6 +584,33 @@ class MeanReciprocalRank(Metric):
         )
 
 
+@dataclass
+class MeanRankConfig(MetricConfig):
+    def to_metric(self):
+        return MeanRank(self)
+
+
+class MeanRank(Metric):
+    """
+    Calculates the mean rank, rank(true_output), the rank of the true output in the
+    predicted n-best list.
+    """
+
+    def mr_val(self, true: Any, preds: list):
+        if true not in preds:
+            return -1  # placeholder for "infinity"; when `true` is not in `preds`
+        else:
+            true_rank = list(preds).index(true) + 1  # 1-indexed
+            return true_rank
+
+    def calc_stats_from_data(
+        self, true_data: list, pred_data: list, config: Optional[MetricConfig] = None
+    ) -> MetricStats:
+        return MetricStats(
+            np.array([self.mr_val(t, p) for t, p in zip(true_data, pred_data)])
+        )
+
+
 class EaaSMetricStats(MetricStats):
     """
     Stats from EaaS for calculation of any of the metrics. These are calculated lazily,
@@ -526,26 +658,24 @@ class EaaSMetricStats(MetricStats):
         return MetricStats(sdata[indices])
 
 
+@dataclass
+class EaaSMetricConfig(MetricConfig):
+    def to_metric(self):
+        return EaaSMetric(self)
+
+
 class EaaSMetric(Metric):
     """
     A metric that calculates evaluation scores using EaaS.
     """
 
-    @classmethod
-    def default_name(cls) -> str:
-        raise NotImplementedError
-
-    def __init__(self, name: str):
-        super().__init__(name)
-        self.name = name
-
     def calc_metric_from_aggregate(
         self, agg_stats: np.ndarray, config: Optional[MetricConfig] = None
     ) -> float:
-        if self.name == 'bleu':
+        if self.config.name == 'bleu':
             bleu_class = sacrebleu.BLEU()
             return bleu_class._compute_score_from_stats(list(agg_stats)).score / 100.0
-        elif self.name == 'chrf':
+        elif self.config.name == 'chrf':
             chrf_class = sacrebleu.CHRF()
             return chrf_class._compute_score_from_stats(list(agg_stats)).score / 100.0
         else:
@@ -557,7 +687,7 @@ class EaaSMetric(Metric):
         :param stats: stats for every example
         :return: aggregated stats
         """
-        if self.name in {'bleu', 'chrf'}:
+        if self.config.name in {'bleu', 'chrf'}:
             return np.sum(stats.get_data(), axis=0)
         else:
             return np.mean(stats.get_data(), axis=0)
@@ -603,30 +733,34 @@ class QAMetric(Metric):
         ...
 
 
+@dataclass
+class ExactMatchQAConfig(MetricConfig):
+    def to_metric(self):
+        return ExactMatchQA(self)
+
+
 class ExactMatchQA(QAMetric):
     """
     Calculate a score for extractive QA based on exact match.
     """
 
-    @classmethod
-    def default_name(cls) -> str:
-        return 'ExactMatchQA'
-
     def sample_level_metric(
         self, ground_truth: str, prediction: str, preprocessor: Preprocessor
     ) -> float:
         return 1.0 if preprocessor(prediction) == preprocessor(ground_truth) else 0.0
 
 
+@dataclass
+class F1ScoreQAConfig(MetricConfig):
+    def to_metric(self):
+        return F1ScoreQA(self)
+
+
 class F1ScoreQA(QAMetric):
     """
     Calculate a score for extractive QA based on F1 score.
     """
 
-    @classmethod
-    def default_name(cls) -> str:
-        return 'F1ScoreQA'
-
     def sample_level_metric(
         self, ground_truth: str, prediction: str, preprocessor: Preprocessor
     ):
@@ -640,3 +774,15 @@ class F1ScoreQA(QAMetric):
         recall = 1.0 * num_same / len(ground_truth_tokens)
         f1 = (2 * precision * recall) / (precision + recall)
         return f1
+
+
+def metric_name_to_config(name: str) -> MetricConfig:
+    try:
+        metric_module = sys.modules[__name__]
+        metric_config = getattr(metric_module, f'{name}Config')
+        return metric_config(name=name)
+    except AttributeError:
+        if name in EndpointConfig().valid_metrics:
+            return EaaSMetricConfig(name=name)
+        else:
+            raise ValueError(f'Invalid metric {name}')
