@@ -1,10 +1,10 @@
 from __future__ import annotations
 
-from explainaboard import feature
+from explainaboard import feature, TaskType
 from explainaboard.info import SysOutputInfo
+from explainaboard.metric import AccuracyConfig, MetricConfig
 from explainaboard.processors.processor import Processor
 from explainaboard.processors.processor_registry import register_processor
-from explainaboard.tasks import TaskType
 from explainaboard.utils.spacy_loader import get_named_entities
 
 
@@ -88,8 +88,8 @@ class AspectBasedSentimentClassificationProcessor(Processor):
         )
 
     @classmethod
-    def default_metrics(cls) -> list[str]:
-        return ["Accuracy"]
+    def default_metrics(cls, language=None) -> list[MetricConfig]:
+        return [AccuracyConfig(name="Accuracy", language=language)]
 
     # --- Feature functions accessible by ExplainaboardBuilder._get_feature_func()
     def _get_sentence_length(self, sys_info: SysOutputInfo, existing_features: dict):
