@@ -2,7 +2,6 @@ from __future__ import annotations
 
 import abc
 from collections.abc import Callable
-import json
 from typing import Any, Optional
 
 from datalabs import aggregating, Dataset, load_dataset
@@ -10,7 +9,7 @@ from eaas.async_client import AsyncClient
 from eaas.config import Config
 from tqdm import tqdm
 
-from explainaboard import feature
+from explainaboard import feature, TaskType
 from explainaboard.info import (
     BucketPerformance,
     FineGrainedStatistics,
@@ -21,11 +20,12 @@ from explainaboard.info import (
     SysOutputInfo,
 )
 from explainaboard.loaders.loader import CustomFeature
-import explainaboard.metric
-from explainaboard.metric import Metric, MetricStats
-from explainaboard.tasks import TaskType
+from explainaboard.metric import Metric, MetricConfig, MetricStats
 import explainaboard.utils.bucketing
-from explainaboard.utils.db_api import read_statistics_from_db, write_statistics_to_db
+from explainaboard.utils.cache_api import (
+    read_statistics_from_cache,
+    write_statistics_to_cache,
+)
 from explainaboard.utils.py_utils import eprint, sort_dict
 from explainaboard.utils.tokenizer import get_default_tokenizer
 from explainaboard.utils.typing_utils import unwrap, unwrap_generator
@@ -49,16 +49,10 @@ class Processor(metaclass=abc.ABCMeta):
     # TODO(gneubig): this could potentially be moved directly into the task definition
     @classmethod
     @abc.abstractmethod
-    def default_metrics(cls) -> list[str]:
+    def default_metrics(cls, language=None) -> list[MetricConfig]:
         """Returns the default metrics of this processor."""
         ...
 
-    @classmethod
-    def default_metric_configs(
-        cls,
-    ) -> Optional[dict[str, explainaboard.metric.MetricConfig]]:
-        return None
-
     def __init__(self) -> None:
         # Things to use only if necessary
         self._eaas_config: Optional[Config] = None
@@ -86,7 +80,6 @@ class Processor(metaclass=abc.ABCMeta):
         """Generate external statistics that are gathered from a relatively costly
         source, such as the training set.
         These are gathered once and then cached for future use.
-
         :param sys_info: Information about the system outputs
         :param statistics_func: The function used to get the statistics
         :return: Statistics from, usually, the training set that are used to calculate
@@ -100,23 +93,23 @@ class Processor(metaclass=abc.ABCMeta):
                 if sys_info.sub_dataset_name == "default"
                 else sys_info.sub_dataset_name
             )
-            try:
-                # read statistics from db
-                message = None
-                if sys_info.reload_stat:
-                    response = read_statistics_from_db(
-                        sys_info.dataset_name, sub_dataset
-                    )
-                    message = json.loads(response.text.replace("null", ""))["message"]
-                    if message == "success":
-                        return json.loads(response.content)['content']
-                # calculate statistics if not reloading or not found
-                if (
-                    not sys_info.reload_stat
-                    or message
-                    == "the dataset does not include the information of _stat"
-                ):
+            # read statistics from cache
+            if sys_info.reload_stat:
+                statistics = read_statistics_from_cache(
+                    sys_info.dataset_name, sub_dataset
+                )
+            if statistics is None:
+                try:
                     dataset = load_dataset(sys_info.dataset_name, sub_dataset)
+                except Exception:
+                    dataset = None
+                if dataset is None:
+                    eprint(
+                        f"{sys_info.dataset_name} hasn't been supported by DataLab so"
+                        " no training set dependent features will be supported by"
+                        " ExplainaBoard. You can add the dataset by: https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sdk.md"  # noqa
+                    )
+                else:
                     self._statistics_func.resources = self._get_statistics_resources(
                         sys_info, dataset[split_name]
                     )
@@ -125,29 +118,14 @@ class Processor(metaclass=abc.ABCMeta):
                         self._statistics_func, mode="local"
                     )
                     statistics = new_train._stat
-                    eprint("saving to database")
-                    response = write_statistics_to_db(
-                        sys_info.dataset_name, sub_dataset, content=statistics
+                    eprint(f"caching stats for {sys_info.dataset_name} {sub_dataset}")
+                    write_statistics_to_cache(
+                        statistics, sys_info.dataset_name, sub_dataset
                     )
-                    eprint(response.content)
-                # dataset does not exist
-                else:
-                    raise FileNotFoundError
-            except FileNotFoundError:
-                eprint(
-                    """
-The dataset hasn't been supported by DataLab so no training set dependent features will
-be supported by ExplainaBoard. You can add the dataset by:
-https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sdk.md
-"""
-                )
         return statistics
 
     def _get_metrics(self, sys_info: SysOutputInfo) -> list[Metric]:
-        return [
-            getattr(explainaboard.metric, name)()
-            for name in unwrap(sys_info.metric_names)
-        ]
+        return [config.to_metric() for config in unwrap(sys_info.metric_configs)]
 
     def _gen_metric_stats(
         self, sys_info: SysOutputInfo, sys_output: list[dict]
@@ -339,7 +317,7 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
         # Bucketing
         samples_over_bucket = {}
         performances_over_bucket = {}
-        for feature_name in tqdm(active_features, desc="bucketing"):
+        for feature_name in tqdm(active_features, desc="sample-level bucketing"):
             # Preparation for bucketing
             bucket_func = getattr(
                 explainaboard.utils.bucketing,
@@ -362,7 +340,6 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
                 samples_over_bucket[feature_name],
                 metric_stats=metric_stats,
             )
-
         return samples_over_bucket, performances_over_bucket
 
     def get_bucket_performance(
@@ -409,8 +386,8 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
                 bucket_samples=bucket_cases,
             )
 
-            for metric_name, metric_func, metric_stat in zip(
-                unwrap_generator(sys_info.metric_names),
+            for metric_cfg, metric_func, metric_stat in zip(
+                unwrap_generator(sys_info.metric_configs),
                 unwrap_generator(metric_funcs),
                 unwrap_generator(metric_stats),
             ):
@@ -427,7 +404,7 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
                 )
 
                 performance = Performance(
-                    metric_name=metric_name,
+                    metric_name=metric_cfg.name,
                     value=metric_result.value,
                     confidence_score_low=conf_low,
                     confidence_score_high=conf_high,
@@ -461,8 +438,8 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
         metric_funcs = self._get_metrics(sys_info)
 
         overall_results = {}
-        for metric_name, metric_func, metric_stat in zip(
-            unwrap_generator(sys_info.metric_names),
+        for metric_cfg, metric_func, metric_stat in zip(
+            unwrap_generator(sys_info.metric_configs),
             unwrap_generator(metric_funcs),
             metric_stats,
         ):
@@ -478,12 +455,12 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
             )
 
             overall_performance = Performance(
-                metric_name=metric_name,
+                metric_name=metric_cfg.name,
                 value=metric_result.value,
                 confidence_score_low=conf_low,
                 confidence_score_high=conf_high,
             )
-            overall_results[metric_name] = overall_performance
+            overall_results[metric_cfg.name] = overall_performance
         return overall_results
 
     def _print_bucket_info(
@@ -512,10 +489,8 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
             metadata["task_name"] = self.task_type().value
 
         sys_info = SysOutputInfo.from_dict(metadata)
-        if sys_info.metric_names is None:
-            sys_info.metric_names = self.default_metrics()
         if sys_info.metric_configs is None:
-            sys_info.metric_configs = self.default_metric_configs()
+            sys_info.metric_configs = self.default_metrics(language=sys_info.language)
         if sys_info.tokenizer is None:
             sys_info.tokenizer = get_default_tokenizer(
                 task_type=self.task_type(), lang=sys_info.language
@@ -541,7 +516,11 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
         return OverallStatistics(sys_info, metric_stats, active_features)
 
     def sort_bucket_info(
-        self, performance_over_bucket, sort_by='value', sort_by_metric='first'
+        self,
+        performance_over_bucket,
+        sort_by='value',
+        sort_by_metric='first',
+        sort_ascending=False,
     ):
         """
         Sorts the `performance_over_bucket` dictionary, which should be of the format
@@ -561,16 +540,17 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
 
         :param sort_by: 'key' or 'value';
             if 'key', sort by the bucket's lower boundary, alphabetically, low-to-high;
-            if 'value', sort by the `value` attribute of the BucketPerformance objects,
-            high-to-low. Please see param sort_by_metric.
+            if 'performance_value', sort by the `value` attribute of the
+            BucketPerformance objects. Since each bucket has multiple metrics
+            associated with it, see param sort_by_metric to choose which metric to
+            sort on.
+            if 'n_bucket_samples', sort by the number of samples in each bucket.
         :param sort_by_metric: 'first' or any string matching the metrics associated
         with this task.
             if 'first', sort by the value of the first BucketPerformance object,
             whichever that may be, high-to-low
             else, sort by the value of that metric.
-            TODO figure out how to let user pass this in.
-        TODO implement another param to specify if they want high-to-low or low-to-high
-        sorting.
+        :param sort_ascending: if True, sort low-to-high; by default, sort high-to-low.
         """
 
         def index_of_metric(metric_bucket_perf_obj, target_metric):
@@ -595,7 +575,7 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
                         reverse=False,
                     )
                 }
-            elif sort_by == 'value':
+            elif sort_by == 'performance_value':
                 if (
                     sort_by_metric == 'first'
                 ):  # sort based on the value of the first feature, whichever that may
@@ -605,7 +585,7 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
                         for k, v in sorted(
                             feature_value.items(),
                             key=lambda item: item[1].performances[0].value,
-                            reverse=True,
+                            reverse=True if not sort_ascending else False,
                         )
                     }
                 else:
@@ -623,9 +603,20 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
                                 )
                             ]
                             .value,
-                            reverse=True,
+                            reverse=True if not sort_ascending else False,
                         )
                     }
+            elif (
+                sort_by == 'n_bucket_samples'
+            ):  # sort by the number of samples in each bucket
+                feature_sorted = {
+                    k: v
+                    for k, v in sorted(
+                        feature_value.items(),
+                        key=lambda item: item[1].n_samples,
+                        reverse=True if not sort_ascending else False,
+                    )
+                }
             performance_over_bucket_sorted[feature_name] = feature_sorted
         return performance_over_bucket_sorted
 
@@ -663,8 +654,9 @@ https://github.com/ExpressAI/DataLab/blob/main/docs/SDK/add_new_datasets_into_sd
             ),  # or 'key' to sort by bucket name, alphabetically
             sort_by_metric=metadata.get(
                 'sort_by_metric', 'first'
-            )  # or whichever metric the user wants.
+            ),  # or whichever metric the user wants.
             # Applicable when sort_by == 'value'
+            sort_ascending=metadata.get('sort_ascending', False),
         )
 
         self._print_bucket_info(performance_over_bucket)
