@@ -1,24 +1,12 @@
 from __future__ import annotations
 
 from dataclasses import dataclass, field
-from enum import Enum
 
-
-class TaskType(str, Enum):
-    text_classification = "text-classification"
-    named_entity_recognition = "named-entity-recognition"
-    question_answering_extractive = "question-answering-extractive"
-    summarization = "summarization"
-    machine_translation = "machine-translation"
-    text_pair_classification = "text-pair-classification"
-    aspect_based_sentiment_classification = "aspect-based-sentiment-classification"
-    kg_link_tail_prediction = "kg-link-tail-prediction"
-    qa_multiple_choice = "qa-multiple-choice"
-    conditional_generation = "conditional-generation"
-
-    @staticmethod
-    def list():
-        return list(map(lambda c: c.value, TaskType))
+from explainaboard import TaskType
+from explainaboard.loaders.loader_registry import (
+    get_supported_file_types_for_loader,
+    SupportedFileFormats,
+)
 
 
 @dataclass
@@ -27,7 +15,9 @@ class Task:
     description: str = "task description"
     supported: bool = field(default=False)
     supported_metrics: list[str] = field(default_factory=list)
-    supported_formats: list[str] = field(default_factory=list)
+    supported_formats: SupportedFileFormats = field(
+        default_factory=SupportedFileFormats
+    )
     supported_datasets: list[str] = field(default_factory=list)
 
 
@@ -66,7 +56,9 @@ _task_categories: list[TaskCategory] = [
                     "mover_score",
                     "prism",
                 ],
-                supported_formats=["tsv"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.machine_translation
+                ),
                 supported_datasets=[],
             ),
             Task(
@@ -93,7 +85,9 @@ https://github.com/neulab/ExplainaBoard/blob/main/docs/task_summarization.md
                     "mover_score",
                     "prism",
                 ],
-                supported_formats=["tsv"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.summarization
+                ),
                 supported_datasets=[],
             ),
             Task(
@@ -119,7 +113,9 @@ https://github.com/neulab/ExplainaBoard/blob/main/docs/task_summarization.md
                     "mover_score",
                     "prism",
                 ],
-                supported_formats=["tsv"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.conditional_generation
+                ),
                 supported_datasets=[],
             ),
         ],
@@ -137,7 +133,9 @@ https://github.com/neulab/ExplainaBoard/blob/main/docs/task_text_classification.
 """,
                 supported=True,
                 supported_metrics=["F1Score", "Accuracy"],
-                supported_formats=["tsv"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.text_classification
+                ),
                 supported_datasets=[],
             )
         ],
@@ -154,10 +152,24 @@ See one example of the uploaded file:
 https://github.com/neulab/ExplainaBoard/blob/main/data/system_outputs/conll2003/conll2003.elmo
 """,
                 supported=True,
-                supported_metrics=["f1_seqeval", "recall_seqeval", "precision_seqeval"],
-                supported_formats=["conll"],
+                supported_metrics=["F1Score"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.named_entity_recognition
+                ),
                 supported_datasets=[],
-            )
+            ),
+            Task(
+                name=TaskType.word_segmentation,
+                description="""
+identify word boundaries of some languages (e.g., Chinese).
+""",
+                supported=True,
+                supported_metrics=["F1Score"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.word_segmentation
+                ),
+                supported_datasets=[],
+            ),
         ],
     ),
     TaskCategory(
@@ -173,7 +185,9 @@ https://github.com/neulab/ExplainaBoard/blob/main/docs/task_extractive_qa_squad.
 """,
                 supported=True,
                 supported_metrics=["F1ScoreQA", "ExactMatchQA"],
-                supported_formats=["json"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.question_answering_extractive
+                ),
                 supported_datasets=[],
             ),
         ],
@@ -191,7 +205,9 @@ https://github.com/neulab/ExplainaBoard/blob/main/data/system_outputs/absa/test-
 """,
                 supported=True,
                 supported_metrics=["F1Score", "Accuracy"],
-                supported_formats=["tsv"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.aspect_based_sentiment_classification
+                ),
                 supported_datasets=[],
             ),
         ],
@@ -209,7 +225,9 @@ https://github.com/neulab/ExplainaBoard/blob/main/docs/task_text_pair_classifica
 """,
                 supported=True,
                 supported_metrics=["F1Score", "Accuracy"],
-                supported_formats=["tsv"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.text_pair_classification
+                ),
                 supported_datasets=[],
             ),
         ],
@@ -226,8 +244,17 @@ See more details about the format of upload files:
 https://github.com/neulab/ExplainaBoard/blob/main/docs/task_kg_link_tail_prediction.md
 """,
                 supported=True,
-                supported_metrics=["Hits", "MeanReciprocalRank"],
-                supported_formats=["json"],
+                supported_metrics=[
+                    "Hit1",
+                    "Hit2",
+                    "Hit3",
+                    "Hit5",
+                    "MeanReciprocalRank",
+                    "MeanRank",
+                ],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.kg_link_tail_prediction
+                ),
                 supported_datasets=[],
             ),
         ],
@@ -241,7 +268,9 @@ https://github.com/neulab/ExplainaBoard/blob/main/docs/task_kg_link_tail_predict
                 description="Answer a question from multiple options",
                 supported=True,
                 supported_metrics=["F1Score", "Accuracy"],
-                supported_formats=["json"],
+                supported_formats=get_supported_file_types_for_loader(
+                    TaskType.qa_multiple_choice
+                ),
                 supported_datasets=[],
             )
         ],
